inst = mc.mcGetInstance()

ModbusMPG = {}

ModbusMPG.mpgLastSelectedAxis = 0
ModbusMPG.lastMPGCount = 0
ModbusMPG.lastMPGMoveCounts = 0

mc.mcMpgSetAccel(inst, 0, 100)  
mc.mcMpgSetRate(inst, 0, 100)
mc.mcMpgSetCountsPerDetent(inst, 0, 4)

local mpgEncoderReg = mc.mcRegGetHandle(inst, "modbus0/MPGEnc")
local xAxisSelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT54)
local yAxisSelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT55)
local zAxisSelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT56)
local aAxisSelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT57)
local bAxisSelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT58)
local cAxisSelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT59)
local mpgEnableReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT60)
local mpgInc1SelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT61)
local mpgInc2SelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT62)
local mpgInc3SelReg = mc.mcSignalGetHandle(inst, mc.ISIG_INPUT63)

local mpgEncoderCountReg = mc.mcRegGetHandle(inst, "gRegs0/MPGCounts")

function setIncs(incVal)
	for i = 0, 5, 1 do 
		mc.mcMpgSetInc(inst, i, incVal)
	end
end

function setMPGAxisAndInc(mpgSelectedAxis)
	local mpgInc1Selected = mc.mcSignalGetState(mpgInc1SelReg)
	local mpgInc2Selected = mc.mcSignalGetState(mpgInc2SelReg)
	local mpgInc3Selected = mc.mcSignalGetState(mpgInc3SelReg)

	mc.mcMpgSetAxis(inst, 0, mpgSelectedAxis - 1)
				
	if mpgInc1Selected == 1 then
		setIncs(0.0001)
	elseif mpgInc2Selected == 2 then
		setIncs(0.001)
	elseif mpgInc3Selected == 1 then 
		setIncs(0.01)
	end
end

function getSelectedMPGAxis()
	local xAxisSelected = mc.mcSignalGetState(xAxisSelReg)
	local yAxisSelected = mc.mcSignalGetState(yAxisSelReg)
	local zAxisSelected = mc.mcSignalGetState(zAxisSelReg)
	local aAxisSelected = mc.mcSignalGetState(aAxisSelReg)
	local bAxisSelected = mc.mcSignalGetState(bAxisSelReg)
	local cAxisSelected = mc.mcSignalGetState(cAxisSelReg)

	local mpgSelectedAxis = 0
	
	if xAxisSelected == 1 then
		mpgSelectedAxis = 1
	elseif yAxisSelected == 1 then
		mpgSelectedAxis = 2
	elseif zAxisSelected == 1 then
		mpgSelectedAxis = 3
	elseif aAxisSelected == 1 then
		mpgSelectedAxis = 4
	elseif bAxisSelected == 1 then
		mpgSelectedAxis = 5
	elseif cAxisSelected == 1 then
		mpgSelectedAxis = 6
	end

	return mpgSelectedAxis
end

function handleAxisSelectionChange(mpgSelectedAxis, mpgEncoderVal)
	if ModbusMPG.mpgLastSelectedAxis ~= tonumber(mpgSelectedAxis) then
		ModbusMPG.lastMPGCount = mpgEncoderVal
		ModbusMPG.lastMPGMoveCounts = mpgEncoderVal
		mc.mcRegSetValueLong(mpgEncoderCountReg, 0)
		ModbusMPG.mpgLastSelectedAxis = tonumber(mpgSelectedAxis)
	end
end

-- a separate global register is used to handle the 16-bit integer rollover of encoder
-- counts. this logic will continue the counts in the + or - direction indefinitely instead
-- of rolling over to max/min size of 16-bit integer once the encoder count extent is reached.
function processMPGMove(mpgEncoderVal)
	local currentMPGEncCounts = mc.mcRegGetValueLong(mpgEncoderCountReg)			
	local mpgCounts = currentMPGEncCounts + math.fmod((mpgEncoderVal - ModbusMPG.lastMPGCount), 32700)
				
	if ModbusMPG.lastMPGCount ~= mpgEncoderVal then
		mc.mcRegSetValueLong(mpgEncoderCountReg, mpgCounts)
		
		mc.mcMpgMoveCounts(inst, 0, mpgCounts - ModbusMPG.lastMPGMoveCounts)
		
		ModbusMPG.lastMPGMoveCounts = mpgCounts
		ModbusMPG.lastMPGCount = mpgEncoderVal
	end
end

function ModbusMPG.RunModbusMPG() 
	local mpgEnabled = mc.mcSignalGetState(mpgEnableReg)
	local mpgEncoderVal = mc.mcRegGetValue(mpgEncoderReg)

	local mpgSelectedAxis = getSelectedMPGAxis()

	if tonumber(mpgSelectedAxis) > 0 and mpgEnabled == 1 then
		setMPGAxisAndInc(mpgSelectedAxis)			
		handleAxisSelectionChange(mpgSelectedAxis, mpgEncoderVal)			
		processMPGMove(mpgEncoderVal)
	end
end

return ModbusMPG